@@ -33,6 +33,7 @@ from typing import Callable, Dict, Iterable, List, Optional, Tuple, Union
 import numpy as np
 import pandas as pd
 from equilibrator_cache import Compound
+from scipy import stats
 from slugify import slugify
 
 from equilibrator_api import Q_, ComponentContribution, R, Reaction, default_T
@@ -47,7 +48,6 @@ class StoichiometricModel(object):
     and magnitudes.
     """
 
-    CONFIDENCE_INTERVAL = 1.96  # TODO: move to params dict
     MINIMAL_STDEV = 1e-3  # TODO: move to params dict
 
     def __init__(
@@ -92,7 +92,6 @@ class StoichiometricModel(object):
             self._bounds = bounds.copy()
 
         self.config_dict = config_dict or dict()
-        self.stdev_factor = 0.0
         self.configure()
 
         # make sure the bounds on Water are (1, 1)
@@ -115,9 +114,11 @@ class StoichiometricModel(object):
             assert standard_dg_primes.check("[energy]/[substance]")
             assert standard_dg_primes.shape == (self.Nr,)
             if dg_sigma is not None:
+                # the format for the uncertainty should be a representation of
+                # the square root of the covariance. note that we need to have
+                # a rank-reduced matrix, so that the number of columns can
+                # also be smaller than Nr
                 assert dg_sigma.check("[energy]/[substance]")
-                # note that the number of columns in the sqrt
-                # of the covariance can be any number, so we don't check it.
                 assert dg_sigma.shape[0] == self.Nr
 
             # dGr should be orthogonal to nullspace of S
@@ -139,18 +140,6 @@ class StoichiometricModel(object):
         # be changed later using the same function (set_compound_ids)
         self.set_compound_ids(compound_id_mapping)
 
-        # Some analyses use Z-score to describe thee distribution of metabolites
-        # so we need to convert the "hard" bounds into a Gaussian distribution
-        # we assume that the given bounds represent the 95% confidence interval
-        # a the Gaussian distribution (i.e. [mu - 1.96*sigma, mu + 1.96*sigma])
-        # Therefore:
-        #              mu = (ub + lb) / 2
-        #           sigma = (ub - lb) / 3.92
-        self.ln_conc_mu = (self.ln_conc_ub + self.ln_conc_lb) / 2.0
-        self.ln_conc_sigma = (self.ln_conc_ub - self.ln_conc_lb) / (
-            self.CONFIDENCE_INTERVAL * 2.0
-        )
-
     def configure(self) -> None:
         """Configure the Component Contribution aqueous conditions."""
         if "p_h" in self.config_dict:
@@ -167,9 +156,27 @@ class StoichiometricModel(object):
         if "temperature" in self.config_dict:
             self.comp_contrib.temperature = Q_(self.config_dict["temperature"])
 
-        # this is the factor by which the STDEV matrix is multiplied to create
-        # the confidence margins for the Gibbs energies of reaction
-        self.stdev_factor = float(self.config_dict.get("stdev_factor", "0"))
+        # dg_confidence is the confidence we want for the bounds on the
+        # standard_dg variables (which is described by a multivariate normal
+        # distribution). 0.95 there is a 5% chance that the correct
+        # solution is outside the bounds. If you don't want to allow any
+        # change to the mean standard_dg estimates, set dg_confidence to 0.
+        if "dg_confidence" in self.config_dict:
+            self.dg_confidence = float(self.config_dict["dg_confidence"])
+        else:
+            self.dg_confidence = 0.95
+
+        # ln_conc_confidence is the confidence we want for the bounds on each
+        # individual log concnetration variables. 0.95 means there is a 5%
+        # chance that the correct solution is outside the bounds (for each
+        # single metabolite indipendently). If you don't want to allow any
+        # change to the mean standard_dg estimates, set ln_conc_confidence to 0.
+        if "ln_conc_confidence" in self.config_dict:
+            self.ln_conc_confidence = float(
+                self.config_dict["ln_conc_confidence"]
+            )
+        else:
+            self.ln_conc_confidence = 0.95
 
     def update_standard_dgs(self) -> None:
         """Calculate the standard ΔG' values and uncertainties.
@@ -180,7 +187,7 @@ class StoichiometricModel(object):
             self.standard_dg_primes,
             self.dg_sigma,
         ) = self.comp_contrib.standard_dg_prime_multi(
-            self.reactions, uncertainty_representation="sqrt"
+            self.reactions, uncertainty_representation="fullrank"
         )
 
     def set_compound_ids(
@@ -202,8 +209,67 @@ class StoichiometricModel(object):
         else:
             self.compound_ids = list(map(mapping, self.S.index))
 
+    def set_bounds(
+        self, compound_id: str, lb: Optional[Q_] = None, ub: Optional[Q_] = None
+    ) -> None:
+        """Set the lower and upper bound of a compound.
+
+        Parameters
+        ----------
+        compound_id: str
+            the compound ID
+        lb: Quantity, optional
+            the new concentration lower bound (ignored if the value is None)
+        ub: Quantity, optional
+            the new concentration upper bound (ignored if the value is None)
+        """
+
+        try:
+            cpd_idx = self.compound_ids.index(compound_id)
+        except ValueError:
+            raise KeyError(f"There is no compound with the ID {compound_id}")
+
+        cpd = self.S.index[cpd_idx]
+        lb = lb or self._bounds.get_lower_bound(cpd)
+        ub = ub or self._bounds.get_upper_bound(cpd)
+
+        assert lb.check(
+            "[concentration]"
+        ), "Bounds must be in units of concentration"
+        assert ub.check(
+            "[concentration]"
+        ), "Bounds must be in units of concentration"
+
+        self._bounds.set_bounds(cpd, lb, ub)
+
+    def get_bounds(self, compound_id: str) -> Tuple[Q_, Q_]:
+        """Get the lower and upper bound of a compound.
+
+        Parameters
+        ----------
+        compound_id: str
+            the compound ID
+
+        Returns
+        -------
+        lb: Quantity, optional
+            the new concentration lower bound (ignored if the value is None)
+        ub: Quantity, optional
+            the new concentration upper bound (ignored if the value is None)
+        """
+
+        try:
+            cpd_idx = self.compound_ids.index(compound_id)
+        except ValueError:
+            raise KeyError(f"There is no compound with the ID {compound_id}")
+
+        cpd = self.S.index[cpd_idx]
+        lb = self._bounds.get_lower_bound(cpd)
+        ub = self._bounds.get_upper_bound(cpd)
+        return lb, ub
+
     @property
-    def bounds(self) -> Tuple[Iterable[float], Iterable[float]]:
+    def bounds(self) -> Tuple[Iterable[Q_], Iterable[Q_]]:
         """Get the concentration bounds.
 
         The order of compounds is according to the stoichiometric matrix index.
@@ -242,6 +308,29 @@ class StoichiometricModel(object):
             list(map(float, self._bounds.get_ln_upper_bounds(self.S.index)))
         )
 
+    @property
+    def ln_conc_mu(self) -> np.array:
+        """Get the mean of the log concentration distribution based on the bounds.
+
+        The order of compounds is according to the stoichiometric matrix index.
+
+        Returns
+        -------
+        a NumPy array with the mean of the log concentrations
+        """
+        return (self.ln_conc_ub + self.ln_conc_lb) / 2.0
+
+    @property
+    def ln_conc_sigma(self) -> np.array:
+        """Get the stdev of the log concentration distribution based on the bounds.
+
+        Returns
+        -------
+        a NumPy array with the stdev of the log concentrations
+        """
+        ci = (self.ln_conc_ub - self.ln_conc_lb) / 2.0
+        return ci / stats.chi2.ppf(self.ln_conc_confidence, 1) ** (0.5)
+
     @staticmethod
     def get_compounds(reactions: Iterable[Reaction]) -> List[Compound]:
         """Get a unique list of all compounds in all reactions.
@@ -482,6 +571,13 @@ class StoichiometricModel(object):
 
         compound_df, bounds_df, reaction_df = dfs
 
+        # remove trailing whitespaces from the ID columns in the compound and
+        # reaction tables, and also in the 'Compound' column of the bounds table
+        # which points to compound IDs
+        compound_df["ID"] = compound_df["ID"].str.strip()
+        reaction_df["ID"] = reaction_df["ID"].str.strip()
+        bounds_df["Compound"] = bounds_df["Compound"].str.strip()
+
         # Read the Compound table
         # -----------------------
         # use equilibrator-cache to build a dictionary of compound IDs to
@@ -500,6 +596,7 @@ class StoichiometricModel(object):
                 raise KeyError(
                     "There is no column of Identifiers in the Compound table"
                 )
+        compound_df["Identifiers"] = compound_df["Identifiers"].str.strip()
 
         comp_contrib = comp_contrib or ComponentContribution()
 
