@@ -104,7 +104,7 @@ class Condition(object):
     I.e. the phase and the abundance.
     """
 
-    def __init__(self, phase: str, abundance: Q_ = None):
+    def __init__(self, phase: str, abundance: ureg.Quantity = None):
         """Create a new condition object."""
         assert phase in PHASE_INFO_DICT, f"Unknown phase: {phase}"
         self._phase = phase
@@ -116,17 +116,17 @@ class Condition(object):
         return self._phase
 
     @property
-    def abundance(self) -> Q_:
+    def abundance(self) -> ureg.Quantity:
         """Return the abundance."""
         return self._abundance
 
     @property
-    def standard_abundance(self) -> Q_:
+    def standard_abundance(self) -> ureg.Quantity:
         """Return the standard abundance in this phase."""
         return PHASE_INFO_DICT[self._phase].standard_abundance
 
     @property
-    def physiolical_abundance(self) -> Q_:
+    def physiolical_abundance(self) -> ureg.Quantity:
         """Return the default physiological abundance in this phase."""
         return PHASE_INFO_DICT[self._phase].physiolical_abundance
 
@@ -163,7 +163,7 @@ class Condition(object):
             ).magnitude
 
     @abundance.setter
-    def abundance(self, abundance: Q_) -> None:
+    def abundance(self, abundance: ureg.Quantity) -> None:
         """Change the phase of a specific compound.
 
         :param abundance: the new abundance in the correct units
@@ -292,12 +292,12 @@ class PhasedCompound(object):
         return NON_AQUEOUS_COMPOUND_DICT.get(self.inchi, (AQUEOUS_PHASE_NAME,))
 
     @property
-    def abundance(self) -> Q_:
+    def abundance(self) -> ureg.Quantity:
         """Get the abundance."""
         return self.condition.abundance
 
     @abundance.setter
-    def abundance(self, abundance: Q_) -> None:
+    def abundance(self, abundance: ureg.Quantity) -> None:
         """Change the phase of a specific compound.
 
         :param abundance: the new abundance in the correct units
@@ -335,8 +335,12 @@ class PhasedCompound(object):
 
     @ureg.check(None, None, "[concentration]", "[temperature]", "")
     def get_stored_standard_dgf_prime(
-        self, p_h: Q_, ionic_strength: Q_, temperature: Q_, p_mg: Q_
-    ) -> Q_:
+        self,
+        p_h: ureg.Quantity,
+        ionic_strength: ureg.Quantity,
+        temperature: ureg.Quantity,
+        p_mg: ureg.Quantity,
+    ) -> ureg.Quantity:
         """Return the stored formation energy of this phased compound.
 
         Only if it exists, otherwise return None (and we will use
@@ -365,7 +369,7 @@ class PhasedCompound(object):
             # for all other compounds, we will use component_contribution
             return None
 
-    def get_stored_standard_dgf(self) -> Q_:
+    def get_stored_standard_dgf(self) -> ureg.Quantity:
         """Return the stored formation energy of this phased compound.
 
         Only if it exists, otherwise return None (and we will use
